/*
 * nrp_tests.c
 *
 * Created: 28-Apr-15 5:36:46 PM
 *  Author: Erlend Riis Jahren
 */ 
#include "nrp_debug.h"
#include "nrp.h"
#include "nrp_error.h"
#include <string.h>
#include "portmacro.h"
#include "FreeRTOS.h"
#include "task.h"
#include "serial/serial.h"
#include "cli/cli.h"
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include "csplib/NRP/nrp_tester.h"
#include <inttypes.h>
#include "nrp_tests.h"


//huge_packet contains 100 bytes of characters
static char *big_packet = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz01";

//huge_packet contains 500 bytes of characters
static const char *huge_packet = "abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789"\
"abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz0123456789abcdefghijklmnopqrstuvwxyz012345";

void test_port_listen_task(void *ptr);

void test_setup_of_port_listener(portCHAR * listen_port){
	uint8_t i = atoi(listen_port);
	int status = nrp_create_port_handler(i, &test_port_listen_task, 1024 + 256, tskIDLE_PRIORITY);
	if (status != pdPASS){
		printf("\rError creating listener: status %i\r\n", status);
	}
}

void test_general(void *ptr){
	char *message = big_packet;
	nuts_module_t to_module = nuts_modules[0];
	
	uint8_t pri;
	if (nuts_modules[this_module].csp_address == 3){
		vTaskDelay(20000);
		pri = CSP_PRIO_CRITICAL;
	}
	else pri = CSP_PRIO_NORM;
	module_socket_t to_socket = {to_module.csp_address, 10};
	module_socket_t from_socket = {nuts_modules[this_module].csp_address, 10};
	nrp_test_t test_s;
	test_s.tot_packet_loss = 0;
	test_s.tot_time = 0;
	test_s.trans_fails = 0;
	uint32_t last_time;
	uint32_t avg_time = 0;
	uint16_t tries = 1000;
	uint32_t time;
	uint32_t fail = 0;
	uint64_t start = xTaskGetTickCount();
	printf("start=%" PRIu64 "\r\n", start);
	for (int i = 0; i < tries + fail; i++){
		last_time = xTaskGetTickCount();
		int8_t status = nrp_send(&to_socket, &from_socket, message, strlen(message), pri);
		if (status < 0){
			fail++;
			time = xTaskGetTickCount() - last_time;
			//printf("Send failed\r\n");
			//printf("s%it%i\r\n", i, 0);
		}
		else {
			time = xTaskGetTickCount() - last_time;
			avg_time += time;
			
			//printf("s%it%i\r\n", i, time);
			printf("s%it%" PRIu64 "\r\n", i, xTaskGetTickCount());
		}
		//if (nuts_modules[this_module].csp_address == 3) vTaskDelay(100);
	}
	printf("total time= %" PRIu64 " sec\r\n", (uint64_t)((xTaskGetTickCount() - start)/1000));
	printf("avg_time of stream: %f, fails: %i\r\n", (float)((float)avg_time / (float)(tries - fail)), fail);
}


/*Listen example task*/
void test_port_listen_task(void *ptr) {
	uint8_t listen_port = *(uint8_t*)ptr;
	
	uint16_t rttt = 0;
	uint32_t trans = 0;
	
	if (nrp_bind_port(listen_port) != SUCCESS){
		vTaskDelete( NULL );
		while(true);
	}
	
	nrp_recv_packet_t * packet;
	
	while(true) {
		packet = nrp_read(listen_port, 1000);
		if (packet == NULL) {
			continue;
		}
		
		if (!compare_payload(huge_packet, packet->data)) printf("\r\n[%i:%i -> %i]: huge_packet received incorrectly (500 bytes)\r\n", packet->src.csp_address, packet->src.port, packet->dest.port);
		
		free_nrp_packet(packet);
	}
}


Bool compare_payload(const uint8_t *recv_payload, const uint8_t *expected_payload){
	for (int i = 0; i < strlen(expected_payload); i++){
		if (recv_payload[i] != recv_payload[i]) {
			return false;
		}
	}
	return true;
}


void test_send_nrp_message(portCHAR * str){
	char *rest = str;
	
	char *message = strsep(&rest, "@");
	uint8_t address = atoi(strsep(&rest, ":"));
	uint8_t port = atoi(rest);
	
	nuts_module_t to_module = nuts_modules[address - 1];
	
	module_socket_t to_socket = {to_module.csp_address, port};
	module_socket_t from_socket = {nuts_modules[this_module].csp_address, 10};
	
	if (strcmp(message, "big") == 0) message = big_packet;
	else if (strcmp(message, "huge") == 0) message = (char*)huge_packet;
	
	
	int8_t status = nrp_send(&to_socket, &from_socket, message, strlen(message), CSP_PRIO_NORM);
	if (status != SUCCESS){
		NRP_PRINT_ERROR("Not able to send message");
	}
}